"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PageVideoCapture = void 0;
const tslib_1 = require("tslib");
const debug_1 = require("debug");
const SortedFrameQueue_1 = require("./SortedFrameQueue");
const ScreencastFrameCollector_1 = require("./ScreencastFrameCollector");
const VideoWriter_1 = require("./VideoWriter");
const debug = debug_1.default('pw-video:PageVideoCapture');
class PageVideoCapture {
    constructor({ collector, queue, page, writer }) {
        this._stopped = false;
        this._collector = collector;
        this._queue = queue;
        this._writer = writer;
        this._writer.on('ffmpegerror', (error) => {
            debug(`stop due to ffmpeg error "${error.trim()}"`);
            this.stop();
        });
        page.on('close', () => this.stop());
        this._listenForFrames();
    }
    static start({ page, savePath, }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            debug('start');
            const collector = yield ScreencastFrameCollector_1.ScreencastFrameCollector.create(page);
            const queue = new SortedFrameQueue_1.SortedFrameQueue();
            const writer = yield VideoWriter_1.VideoWriter.create(savePath);
            const capture = new PageVideoCapture({ collector, queue, page, writer });
            yield collector.start();
            return capture;
        });
    }
    _listenForFrames() {
        this._collector.on('screencastframe', (screencastFrame) => {
            debug(`collected frame from screencast: ${screencastFrame.timestamp}`);
            this._queue.insert(screencastFrame);
        });
        this._queue.on('sortedframes', (frames) => {
            debug(`received ${frames.length} frames from queue`);
            frames.forEach((frame) => this._writePreviousFrame(frame));
        });
    }
    _writePreviousFrame(currentFrame) {
        if (this._previousFrame) {
            const durationSeconds = (currentFrame.received - this._previousFrame.received) / 1000;
            this._writer.write(this._previousFrame.data, durationSeconds);
        }
        this._previousFrame = currentFrame;
    }
    _writeFinalFrame() {
        if (!this._previousFrame)
            return;
        debug('write final frame');
        const durationSeconds = (Date.now() - this._previousFrame.received) / 1000;
        this._writer.write(this._previousFrame.data, durationSeconds);
    }
    stop() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this._stopped)
                return;
            debug('stop');
            this._stopped = true;
            yield this._collector.stop();
            this._queue.drain();
            this._writeFinalFrame();
            return this._writer.stop();
        });
    }
}
exports.PageVideoCapture = PageVideoCapture;
//# sourceMappingURL=PageVideoCapture.js.map